'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reselect = require('reselect');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function paginate(rows, _ref) {
  var page = _ref.page;
  var pageSize = _ref.pageSize;

  if (pageSize < 1) {
    return rows.slice(0);
  }
  var start = page * pageSize;
  return rows.slice(start, start + pageSize);
}

function filter() {
  var rows = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];
  var textFilter = arguments[1];
  var columns = arguments[2];

  if (!textFilter) {
    return rows.slice(0);
  }
  return _lodash2.default.filter(rows, function (row) {
    return _lodash2.default.some(columns, function (column) {
      if (!column.filterable) {
        return false;
      }
      var normalized = String(_lodash2.default.get(row, column.key)).toLowerCase();
      return normalized.indexOf(textFilter) > -1;
    });
  });
}

function sort(rows, _ref2) {
  var sortKey = _ref2.sortKey;
  var direction = _ref2.direction;

  var cloned = rows.slice(0);
  if (!sortKey) {
    return cloned;
  }
  return cloned.sort(function (a, b) {
    var sortVal = 0;
    if (_lodash2.default.get(a, sortKey) > _lodash2.default.get(b, sortKey)) {
      sortVal = 1;
    }
    if (_lodash2.default.get(a, sortKey) < _lodash2.default.get(b, sortKey)) {
      sortVal = -1;
    }
    if (direction === 'asc') {
      sortVal *= -1;
    }
    return sortVal;
  });
}

// wrapped in function as we use the same selectors for multiple tables
// if we don't wrap selectors like this, they would never memoize/cache results
// as we use it for multiple tables (each table has different state)

exports.default = function (tableName) {
  var tableProp = function tableProp(state, prop) {
    return state.sematable[tableName] ? _lodash2.default.get(state.sematable[tableName], prop) : undefined;
  };

  var getIsInitialized = function getIsInitialized(state) {
    return state.sematable[tableName] !== undefined;
  };
  var getInitialData = function getInitialData(state) {
    return tableProp(state, 'initialData');
  };
  var getFilter = function getFilter(state) {
    return tableProp(state, 'filter');
  };
  var getColumns = function getColumns(state) {
    return tableProp(state, 'columns');
  };
  var getPage = function getPage(state) {
    return tableProp(state, 'page');
  };
  var getPrimaryKey = function getPrimaryKey(state) {
    return tableProp(state, 'primaryKey');
  };
  var getPageSize = function getPageSize(state) {
    return tableProp(state, 'pageSize');
  };
  var getUserSelection = function getUserSelection(state) {
    return tableProp(state, 'userSelection');
  };
  var getSelectAll = function getSelectAll(state) {
    return tableProp(state, 'selectAll');
  };
  var getSortInfo = function getSortInfo(state) {
    return {
      sortKey: tableProp(state, 'sortKey'),
      direction: tableProp(state, 'direction')
    };
  };
  var getSelectEnabled = function getSelectEnabled(state) {
    return tableProp(state, 'configs.selectEnabled');
  };

  var getFiltered = (0, _reselect.createSelector)(getInitialData, getFilter, getColumns, function (initialData, textFilter, columns) {
    return filter(initialData, textFilter, columns);
  });

  var getPageInfo = (0, _reselect.createSelector)(getPage, getPageSize, getFiltered, function (page, pageSize, filtered) {
    if (pageSize === -1) {
      // we are showing all rows
      return {
        page: page,
        pageSize: pageSize,
        pageCount: 1
      };
    }
    var pageCount = Math.ceil(filtered.length / pageSize);
    // When initial data changes (and therefore filtered data), we might have
    // less data than before. If that's the case the current page value might
    // be invalid. We fix that by setting it to last page.
    var validPage = page;
    if (page > pageCount - 1) {
      validPage = pageCount - 1;
    }
    return {
      page: validPage,
      pageSize: pageSize,
      pageCount: pageCount
    };
  });

  var getSorted = (0, _reselect.createSelector)(getFiltered, getSortInfo, function (filtered, sortInfo) {
    return sort(filtered, sortInfo);
  });

  var getVisible = (0, _reselect.createSelector)(getSorted, getPageInfo, function (sorted, pageInfo) {
    return paginate(sorted, pageInfo);
  });

  var getSelectedRows = (0, _reselect.createSelector)(getFiltered, getColumns, getUserSelection, getSelectAll, getPrimaryKey, getSelectEnabled, function (filtered, columns, userSelection, selectAll, primaryKey, selectEnabled) {
    var includesKey = function includesKey(row) {
      return _lodash2.default.includes(userSelection, _lodash2.default.get(row, primaryKey));
    };

    if (selectAll) {
      var selectable = filtered;
      // if not all rows are selectable, apply selectEnabled function to filter selectable
      if (selectEnabled) {
        selectable = _lodash2.default.filter(selectable, selectEnabled);
      }
      if (_lodash2.default.isEmpty(userSelection)) {
        return selectable;
      }
      // when select all is enabled, userSelection acts as "not selected" rows
      return _lodash2.default.reject(selectable, includesKey);
    }

    // when select all is not enabled, userSelection acts as "selected" rows
    return _lodash2.default.filter(filtered, includesKey);
  });

  return {
    getInitialData: getInitialData,
    getIsInitialized: getIsInitialized,
    getFilter: getFilter,
    getColumns: getColumns,
    getSortInfo: getSortInfo,
    getPageInfo: getPageInfo,
    getVisible: getVisible,
    getSelectedRows: getSelectedRows,
    getSelectAll: getSelectAll,
    getPrimaryKey: getPrimaryKey
  };
};